create or replace procedure pr_Create_CatCodes(
    v_nbrcusts1     IN  integer,
    v_nbrcusts2     IN  integer,
    v_nbritems1     IN  integer,
    v_nbritems2     IN  integer) 
--
--  Jim Maydew  ISMG 6840  6/29/01 - Inventory DW
--  This procedure creates the requested number of category codes
--  for customers/vendors and items.
--  
is
    v_i        integer;
    v_count    integer;
    v_cat      varchar2(4);
    v_adesc1   addr_cat_code1.AddrCatDesc%type;
    v_idesc1   item_cat_code1.ItemCatDesc%type;

begin
--
--  verify input parms
--
   if (v_nbrcusts1 > 99 or v_nbrcusts2 > 99) then
      raise_application_error(-20001,'Error - Input customer category code creation parameter(s) values must be < 100');
   end if;
   if (v_nbritems1 > 99 or v_nbritems2 > 99) then
      raise_application_error(-20002,'Error - Input item category code creation parameter(s) values must be < 100');
   end if;
   if (v_nbrcusts1 <= 0 or v_nbrcusts2 <= 0) then
      raise_application_error(-20003,'Error - Input customer category code creation parameter(s) values must be greater than zero');
   end if;
   if (v_nbritems1 <= 0 or v_nbritems2 <= 0) then
      raise_application_error(-20004,'Error - Input item category code creation parameter(s) values must be greater than zero');
   end if;
--
--  First see if addr category code table is empty
--
   select count(*) into v_count from addr_cat_code1;
   if (v_count <> 0) then
      raise_application_error(-20005,'Error - Table (addr_cat_code1) must be empty - '
       || to_char(v_count) || ' rows exist');
   end if;
   select count(*) into v_count from addr_cat_code2;
   if (v_count <> 0) then
      raise_application_error(-20006,'Error - Table (addr_cat_code2) must be empty - '
       || to_char(v_count) || ' rows exist');
   end if;
--
--  Next see if item category code table is empty
--
   select count(*) into v_count from item_cat_code1;
   if (v_count <> 0) then
      raise_application_error(-20007,'Error - Table (item_cat_code1) must be empty'
       || to_char(v_count) || ' rows exist');
   end if;
   select count(*) into v_count from item_cat_code2;
   if (v_count <> 0) then
      raise_application_error(-20008,'Error - Table (item_cat_code2) must be empty'
       || to_char(v_count) || ' rows exist');
   end if;
--
--  insert rows into addr cat code table based on input parm
--
   for v_i in 1..v_nbrcusts1 loop
      v_cat := 'AA' || to_char(v_i);
      v_adesc1 := 'Addr Cat Code ' || v_cat;
      insert into addr_cat_code1 values (addr_cat_code1_seq.nextval, v_cat, v_adesc1);
   end loop;
   dbms_output.put_line(' ' || to_char(v_nbrcusts1) || ' rows added to the addr_cat_code1 table');
--
   for v_i in 1..v_nbrcusts2 loop
      v_cat := 'AB' || to_char(v_i);
      v_adesc1 := 'Addr Cat Code ' || v_cat;
      insert into addr_cat_code2 values (addr_cat_code2_seq.nextval, v_cat, v_adesc1);
   end loop;
   dbms_output.put_line(' ' || to_char(v_nbrcusts2) || ' rows added to the addr_cat_code2 table');
--
--  insert rows into item cat code table based on input parm
--
   for v_i in 1..v_nbritems1 loop
      v_cat := 'PA' || to_char(v_i);
      v_idesc1 := 'Item Cat Code ' || v_cat;
      insert into item_cat_code1 values (item_cat_code1_seq.nextval, v_cat, v_idesc1);
   end loop;
   dbms_output.put_line(' ' || to_char(v_nbritems1) || ' rows added to the item_cat_code1 table');
--
   for v_i in 1..v_nbritems2 loop
      v_cat := 'PB' || to_char(v_i);
      v_idesc1 := 'Item Cat Code ' || v_cat;
      insert into item_cat_code2 values (item_cat_code2_seq.nextval, v_cat, v_idesc1);
   end loop;
   dbms_output.put_line(' ' || to_char(v_nbritems2) || ' rows added to the item_cat_code2 table');
end;
/

create or replace procedure pr_Create_companies(
    v_nbrcomp         IN  integer,         -- number of companies to create
    v_nbrbranches     IN  integer,         -- number of branch plants/company to create
    v_carCost         in  branch_plant_dim.CarryingCost%type,  -- max random carrying cost
    v_costMeth        in  branch_plant_dim.CostMethod%type) -- cost methods 01 - 07
--
--  Jim Maydew  ISMG 6840  6/29/01 - Inventory DW
--  This procedure creates the requested number of companies
--  and branches per company.
--  
is
    v_i        integer;
    v_j        integer;
    v_count    integer;
    v_random   integer;
    v_compId   company_dim.CompanyKey%type;
    v_branchId branch_plant_dim.BranchPlantKey%type;
    v_ccost    branch_plant_dim.CarryingCost%type;
    v_cname    company_dim.CompanyName%type;
    v_bpname   branch_plant_dim.BPName%type;

begin
--
--  verify input parms
--
   if (v_nbrcomp > 5 or v_nbrbranches > 10) then
      raise_application_error(-20011,'Error - Input parameter values must be less than 5 companies, 10 branches');
   end if;
   if (v_nbrcomp <= 0 or v_nbrbranches <= 0) then
      raise_application_error(-20012,'Error - Input parameter values must be greater than zero');
   end if;
--
--  First see if company table is empty
--
   select count(*) into v_count from company_dim;
   if (v_count <> 0) then
      raise_application_error(-20013,'Error - Table (company_dim) must be empty - '
       || to_char(v_count) || ' rows exist');
   end if;
--
--  Next see if branch table is empty
--
   select count(*) into v_count from branch_plant_dim;
   if (v_count <> 0) then
     raise_application_error(-20014,'Error - Table (branch_plant_dim) must be empty'
       || to_char(v_count) || ' rows exist');
   end if;
--
--  insert rows into company and branch plant based on cat code table based on input parm
--
    dbms_random.initialize(1234567);
    for v_i in 1..v_nbrcomp loop  
       select company_seq.nextval into v_compId from dual;
       v_cname := 'Company ' || to_char(v_compId);
       insert into company_dim values (v_compId, to_char(v_compId), v_cname, 'USD', 'US Dollars');
       dbms_output.put_line(' Company ' || to_char(v_compId) || ' Added ');
       for v_j in 1..v_nbrbranches loop
          v_random := mod(abs(dbms_random.random), round(100 * v_carCost,0));
          v_ccost := round(v_random / 100, 2);
          select branch_plant_seq.nextval into v_BranchId from dual;
          v_bpname := 'Branch Plant ' || to_char(v_BranchId);
          insert into branch_plant_dim values (v_BranchId, to_char(v_BranchId), v_i, v_ccost, v_costMeth, v_bpname);
          dbms_output.put_line(' Branch Plant ' || to_char(v_BranchId) || ' added to Company ' || to_char(v_compId));
        end loop;
    end loop;
end;
/

create or replace procedure pr_Create_CustVendor(
    v_nbrcusts    IN  integer)         

--
--  Jim Maydew  ISMG 6840  6/29/01 - Inventory DW
--  This procedure creates the cust_vendor_dim table
--  
is
    v_count      integer;
    v_cc_count1  integer;  -- number of addr1 cat codes
    v_cc_count2  integer;  -- number of addr2 cat codes
    v_zip_count  integer;  -- number of zip rows
    v_i          integer;  -- loop counter
    v_j          integer;  -- loop counter
    v_nbrzips    integer;  -- count of approx number of zips to generate per zip_code row...
    v_random     integer;  -- random number
    v_CustKey    cust_vendor_dim.CustVendorKey%type;
    v_name       cust_vendor_dim.Name%type;
    v_Addr       cust_vendor_dim.Address%type;
    v_pzip       cust_vendor_dim.primZip%type;
    v_zip        cust_vendor_dim.zip%type;
    v_ctry       cust_vendor_dim.Country%type := 'USA';
    v_cc1        cust_vendor_dim.AddrCatCode1%type;
    v_cc2        cust_vendor_dim.AddrCatCode2%type;
    v_zcity      zip_codes.ZipCity%type;
    v_zstate     zip_codes.ZipState%type;
    v_zzip       zip_codes.ZipZip%type;
    v_zconsec    zip_codes.ZipConsec%type;
    v_zweight    zip_codes.ZipWeight%type;
begin
--
--  verify input parms
--
   if (v_nbrcusts > 10000) then
      raise_application_error(-20041,'Error - Input parameter values must be less than 10,000 customers');
   end if;
   if (v_nbrcusts <= 0) then
      raise_application_error(-20042,'Error - Input parameter values must be greater than zero');
   end if;
--
--  First see if customer table is empty
--
   select count(*) into v_count from cust_vendor_dim;
   if (v_count <> 0) then
      raise_application_error(-20043,'Error - Table (cust_vendor_dim) must be empty - '
       || to_char(v_count) || ' rows exist');
   end if;
--
--  get count of category codes
--
   select count(*) into v_cc_count1 from addr_cat_code1;
   if (v_cc_count1 = 0) then
      raise_application_error(-20044,'Error - Table (addr_cat_code1) must NOT be empty - '
       || to_char(v_cc_count1) || ' rows exist');
   end if;
--
   select count(*) into v_cc_count2 from addr_cat_code2;
   if (v_cc_count2 = 0) then
      raise_application_error(-20045,'Error - Table (addr_cat_code2) must NOT be empty - '
       || to_char(v_cc_count2) || ' rows exist');
   end if;
--
--  get count of zip code table
   select count(*) into v_zip_count from zip_codes;
   if (v_zip_count = 0) then
      raise_application_error(-20046,'Error - Table (zip_codes) must NOT be empty - '
       || to_char(v_zip_count) || ' rows exist');
   end if;
--
--  insert rows into cust_vendor_dim table
--
   v_count :=1;   
   v_j := 1;
--
--  figure out city, state and zip based on zip codes table
--  the zip is calculated based on consec zips with a random number...
--
   select ZipCity, ZipState, ZipZip, ZipConsec, ZipWeight
        into v_zcity, v_zstate, v_zzip, v_zconsec, v_zweight
        from zip_codes
        where ZipKey = v_count;
   v_nbrzips := trunc(v_nbrcusts * v_zweight / 100);
   v_pzip := v_zzip + mod(abs(dbms_random.random), v_zconsec);
--
   dbms_random.initialize(1234567); 
   for v_i in 1..v_nbrcusts loop
      v_cc1 := mod(abs(dbms_random.random), v_cc_count1) + 1;
      v_cc2 := mod(abs(dbms_random.random), v_cc_count2)+ 1;
      v_random := mod(abs(dbms_random.random), 100000)+ 1;
      select cust_vendor_seq.nextval into v_CustKey from dual;
      v_name := 'Customer ' || to_char(v_CustKey);
      v_addr := to_char(v_random) || ' Some Street';
      v_zip := to_char(v_pzip);
      if (length(v_zip) = 4) then
        v_zip := '0' || v_zip;
      end if;
      insert into cust_vendor_dim values (v_CustKey, v_CustKey, v_name, v_Addr, v_zcity, v_zstate, 
                 v_pzip, v_zip, v_ctry, v_cc1, v_cc2);
      v_j := v_j + 1;
-- 
--  decide if next zip code record needs to be fetched...
-- 
      if (v_j >= v_nbrzips) then
        v_j :=1;
        v_count := v_count + 1;
        if (v_count > v_zip_count) then
           v_count :=1;
        end if;
        select ZipCity, ZipState, ZipZip, ZipConsec, ZipWeight
             into v_zcity, v_zstate, v_zzip, v_zconsec, v_zweight
             from zip_codes
             where ZipKey = v_count;
        v_nbrzips := trunc(v_nbrcusts * v_zweight / 100);
        v_pzip := v_zzip + mod(abs(dbms_random.random), v_zconsec);
      else 
        v_nbrzips := trunc(v_nbrcusts * v_zweight / 100) + 1;
        v_pzip := v_zzip + mod(abs(dbms_random.random), v_zconsec);
      end if;
      if (mod(v_i, trunc(v_nbrcusts/10)) = 0) then
         dbms_output.put_line(' ' || to_char(v_i) || ' Customers/Vendors added to cust_vendor_table ');
      end if;
   end loop;
   dbms_output.put_line(' ' || to_char(v_nbrcusts) || ' TOTAL Customers/Vendors added to cust_vendor_table ');
end;
/

create or replace procedure pr_Create_dates(
    v_mode            IN  integer,
    v_jdate           IN  varchar2,
    v_nbrdates        IN  integer)         -- number of items to create
--
--  Jim Maydew  ISMG 6840  6/29/01 - Inventory DW
--  This procedure creates the requested number of dates
--  
is
    v_i           integer;  -- loop variable
    v_j           integer;  -- loop variable
    v_jend        integer;  -- end loop variable
    v_monadd      integer;  -- month amount to add
    v_count       integer;  -- number of rows in table
    v_tempjd      integer;  -- temp julian date
    v_jyear       integer;  -- year yyyy
    v_jday        integer;  -- julian day ddd
    v_leapyr      integer;
    v_tempdt      date;
    v_stempdt     varchar(10);  -- mm/dd/yyyy
    v_dayofweek   varchar2(3);
    v_DateKey     date_dim.DateKey%type;
    v_datej       date_dim.DateJulian%type;
    v_calday      date_dim.CalDay%type;
    v_calmon      date_dim.CalMonth%type;
    v_calqtr      date_dim.CalQuarter%type;
    v_calyear     date_dim.CalYear%type;
    v_dow         date_dim.DayOfWeek%type;
    v_fyear       date_dim.FiscalYear%type;
    v_fper        date_dim.FiscalPeriod%type;

begin
--
--  verify mode parms and length of julian date
--
   if (v_mode > 4 or v_mode < 1) then
      raise_application_error(-20051,'Error - Mode must be less than 5');
   end if;
   if (length(rtrim(v_jdate)) <> 7) then
      raise_application_error(-20052,'Error - Julian date parameter must be in 7 characters in length');
   end if;
   v_jyear := to_number(substr(v_jdate,1,4));
   v_jday := to_number(substr(v_jdate,5,3));
--
--  verify julian date parms
--
   if (v_jyear < 1900 or v_jyear > 2100) then
      raise_application_error(-20053,'Error - Julian Date Year must be <= 2100 and => 1900');
   end if;
   if (v_jday = 366 and mod(v_jyear,4) <> 0) then
      raise_application_error(-20054,'Error - Julian year is not a leap year, days <> 366');
   end if;
   if (v_jday < 1 or v_jday > 366) then
      raise_application_error(-20055,'Error - Julian date Day must be <= 365 and => 1');
   end if;
--
--  verify number of dates parm
--
   if (v_nbrdates < 1 or v_nbrdates > 2000) then
      raise_application_error(-20056,'Error - number of generated dates must be <= 2000');
   end if;
--
--  First see if date table is empty
--
   select count(*) into v_count from date_dim;
   if (v_count <> 0) then
      raise_application_error(-20056,'Error - Table (date_dim) must be empty - '
       || to_char(v_count) || ' rows exist');
   end if;
--
--  insert rows into date_dim based on input parms
--
    v_tempjd := v_jdate;
    for v_i in 1..v_nbrdates loop  
       select date_seq.nextval into v_DateKey from dual;
       v_datej := v_tempjd;
-- figure out if working date is a leap year
       if (mod(v_jyear, 4) = 0) then
          v_leapyr := 1;
       else
          v_leapyr := 0;
       end if;
--
-- calculate day of month and month based on julian days
--
       if (v_jday <= 31) then
          v_calday := v_jday;
          v_calmon := 1;  
          v_monadd := 31;
       elsif (v_jday <= (59 + v_leapyr)) then
          v_calday := v_jday - 31;
          v_calmon := 2;
          v_monadd := 28 + v_leapyr;
        elsif (v_jday <= (90 + v_leapyr)) then
          v_calday := v_jday - 59 - v_leapyr;
          v_calmon := 3;
          v_monadd := 31;
        elsif (v_jday <= (120 + v_leapyr)) then  
          v_calday := v_jday - 90 - v_leapyr;
          v_calmon := 4;          
          v_monadd := 30;
        elsif (v_jday <= (151 + v_leapyr)) then
          v_calday := v_jday - 120 - v_leapyr;
          v_calmon := 5;
          v_monadd := 31;
        elsif (v_jday <= (181 + v_leapyr)) then  
          v_calday := v_jday - 151 - v_leapyr;
          v_calmon := 6;
          v_monadd := 30;
        elsif (v_jday <= (212 + v_leapyr)) then  
          v_calday := v_jday - 181 - v_leapyr;
          v_calmon := 7;
          v_monadd := 31;
        elsif (v_jday <= (243 + v_leapyr)) then  
          v_calday := v_jday - 212 - v_leapyr;
          v_calmon := 8;
          v_monadd := 31;
        elsif (v_jday <= (273 + v_leapyr)) then  
          v_calday := v_jday - 243 - v_leapyr;
          v_calmon := 9;
          v_monadd := 30;
        elsif (v_jday <= (304 + v_leapyr)) then   
          v_calday := v_jday - 273 - v_leapyr;
          v_calmon := 10;
          v_monadd := 31;
        elsif (v_jday <= (334 + v_leapyr)) then  
          v_calday := v_jday - 304 - v_leapyr;
          v_calmon := 11;
          v_monadd := 30;
        else   
          v_calday := v_jday - 334 - v_leapyr;
          v_calmon := 12;
          v_monadd := 0;
        end if;
--
-- calculate the quarter
--
       if (v_jday < 91 + v_leapyr) then
          v_calqtr :=1;
       elsif (v_jday < 182 + v_leapyr) then
          v_calqtr :=2;
       elsif (v_jday < 274 + v_leapyr) then
          v_calqtr :=3;
       else
          v_calqtr :=4;
       end if;
--
--  use julian year
--
       v_calyear := v_jyear;
--
--  calculate the day of the week...
--
       v_stempdt := to_char(v_calmon) || '/' || to_char(v_calday) || '/' || to_char(v_calyear);
       v_tempdt := to_date(v_stempdt, 'MM/DD/YYYY');
       v_dayofweek := to_char (v_tempdt, 'DY');
       if (v_dayofweek = 'SUN') then
          v_dow := 0;
       elsif (v_dayofweek = 'MON') then
          v_dow := 1;
       elsif (v_dayofweek = 'TUE') then
          v_dow := 2;
       elsif (v_dayofweek = 'WED') then
          v_dow := 3;
       elsif (v_dayofweek = 'THU') then
          v_dow := 4;
       elsif (v_dayofweek = 'FRI') then
          v_dow := 5;
       elsif (v_dayofweek = 'SAT') then
          v_dow := 6;
       else
          v_dow := 0;
       end if;
      
-- 
-- assume fiscal year = calendar year, fiscal period = calendar month...
--
       v_fyear := v_jyear;
       v_fper := v_calmon;
       insert into date_dim values (v_datekey, v_datej, v_calday, v_calmon, v_calqtr, v_calyear, v_dow, v_fyear, v_fper);
       if (mod(v_i, trunc(v_nbrdates/10)) = 0) then
          dbms_output.put_line(' ' || to_char(v_i) || ' dates added to date_dim table ');
       end if;
--
--  Calculate next time period...
--
       if (v_mode = 1) then
          if (v_jday + 1 > 365 + v_leapyr) then
             v_jday :=1;
             v_jyear := v_jyear + 1;
             v_tempjd := (v_jyear * 1000) + v_jday;
          else
             v_tempjd := v_tempjd + 1;
             v_jday := v_jday + 1;
          end if;
        elsif (v_mode = 2 or v_mode = 3) then
          if (v_mode =2 and mod(v_i,5) = 0) then
             v_jend := 3;
          elsif (v_mode = 2) then
             v_jend := 1;
          elsif (v_mode = 3) then
             v_jend := 7;
          else
             v_jend := 1;
          end if;
--
-- skip next v_jend days...
--
          for v_j in 1..v_jend loop
             if (v_jday + 1 > 365 + v_leapyr) then
                v_jday :=1;
                v_jyear := v_jyear + 1;
                v_tempjd := (v_jyear * 1000) + v_jday;
             else
                v_tempjd := v_tempjd + 1;
                v_jday := v_jday + 1;
             end if;
          end loop;
--
-- mode is monthly
--
        else 
          if (v_monadd = 0) then
             v_jday := v_calday;
             v_jyear := v_jyear + 1;
             v_tempjd := (v_jyear * 1000) + v_jday;
          else
             v_tempjd := v_tempjd + v_monadd;
             v_jday := v_jday + v_monadd;
          end if;
       end if;
    end loop;   
    dbms_output.put_line(' ' || to_char(v_nbrdates) || ' TOTAL dates added to date_dim table ');
end;
/

create or replace procedure pr_Create_items(
    v_nbrparts        IN  integer)         -- number of items to create
--
--  Jim Maydew  ISMG 6840  6/29/01 - Inventory DW
--  This procedure creates the requested number of part numbers
--  
is
    v_i          integer;
    v_count      integer;
    v_cc_count1  integer;
    v_cc_count2  integer;
    v_itemId     item_master_dim.ItemMasterKey%type;
    v_2nd        item_master_dim.SecondItemId%type;
    v_3rd        item_master_dim.ThirdItemId%type;
    v_cc1        item_master_dim.ItemCatCode1%type;
    v_cc2        item_master_dim.ItemCatCode2%type;
    v_desc       item_master_dim.ItemDesc%type;
    v_uom        item_master_dim.UOM%type := 'EA';

begin
--
--  verify input parms
--
   if (v_nbrparts > 10000) then
      raise_application_error(-20021,'Error - Input parameter values must be less than 10,000 items');
   end if;
   if (v_nbrparts <= 0) then
      raise_application_error(-20022,'Error - Input parameter values must be greater than zero');
   end if;
--
--  First see if item table is empty
--
   select count(*) into v_count from item_master_dim;
   if (v_count <> 0) then
      raise_application_error(-20023,'Error - Table (item_master_dim) must be empty - '
       || to_char(v_count) || ' rows exist');
   end if;
--
--  get count of item category code1
--
   select count(*) into v_cc_count1 from item_cat_code1;
   if (v_cc_count1 = 0) then
      raise_application_error(-20024,'Error - Table (item_cat_code1) must NOT be empty - '
       || to_char(v_cc_count1) || ' rows exist');
   end if;
--
   select count(*) into v_cc_count2 from item_cat_code2;
   if (v_cc_count2 = 0) then
      raise_application_error(-20025,'Error - Table (item_cat_code2) must NOT be empty - '
       || to_char(v_cc_count2) || ' rows exist');
   end if;
--
--  insert rows into item_master based on input parm
--
    dbms_random.initialize(1234567);
    for v_i in 1..v_nbrparts loop  
       v_cc1 := mod(abs(dbms_random.random), v_cc_count1) + 1;
       v_cc2 := mod(abs(dbms_random.random), v_cc_count2)+ 1;
       select item_master_seq.nextval into v_ItemId from dual;
       v_2nd := 'Second Part ' || to_char(v_ItemId);
       v_3rd := 'Thrid Part ' || to_char(v_ItemId);
       v_desc := 'Part Description ' || to_char(v_ItemId);
       insert into item_master_dim values (v_ItemId, v_ItemId, v_2nd, v_3rd, v_cc1, v_cc2, v_desc, v_uom);
       if (mod(v_i, trunc(v_nbrparts/10)) = 0) then
          dbms_output.put_line(' ' || to_char(v_i) || ' parts added to item_master_table ');
       end if;
    end loop;   
    dbms_output.put_line(' ' || to_char(v_nbrparts) || ' TOTAL parts added to item_master_table ');
end;
/

create or replace procedure pr_Create_TransType         

--
--  Jim Maydew  ISMG 6840  6/29/01 - Inventory DW
--  This procedure creates the transaction type table
--  
is
    v_count      integer;
begin
--
--  First see if transaction type table is empty
--
   select count(*) into v_count from trans_type_dim;
   if (v_count <> 0) then
      raise_application_error(-20031,'Error - Table (trans_type_dim) must be empty - '
       || to_char(v_count) || ' rows exist');
   end if;
--
--  insert rows into transaction type table
--
   insert into trans_type_dim values (1, 'IA', 'Inventory Adjustement');
   insert into trans_type_dim values (2, 'IT', 'Inventory Transfer');
   insert into trans_type_dim values (3, 'IS', 'Inventory Simple Issue');
   insert into trans_type_dim values (4, 'OV', 'Purchase Order Receipt');
   insert into trans_type_dim values (5, 'AR', 'Sales Order Shipment');
-- insert into trans_type_dim values (6, 'IM', 'Manufacturing Issue');
-- insert into trans_type_dim values (7, 'IC', 'Manufacturing Completion');
-- insert into trans_type_dim values (8, 'IS', 'Manufacturing Assembly Scrap');
-- insert into trans_type_dim values (9, 'IZ', 'Manufacturing Component Scrap');
   dbms_output.put_line('5 Transaction Type rows added to trans_type_dim table');
end;
/

create or replace procedure pr_Create_ZipCodes         

--
--  Jim Maydew  ISMG 6840  6/29/01 - Inventory DW
--  This procedure populates the zip_codes table...
--  
is
    v_count      integer;
begin
--
--  First see if zip_codes table is empty
--
   select count(*) into v_count from zip_codes;
   if (v_count <> 0) then
      raise_application_error(-20061,'Error - Table (zip_codes) must be empty - '
       || to_char(v_count) || ' rows exist');
   end if;
--
--  insert rows into transaction type table
--
   insert into zip_codes values (1, 'Chicago', 'IL', 60601, 398, 10);
   insert into zip_codes values (2, 'New York City', 'NY', 10001, 292, 10);
   insert into zip_codes values (3, 'Denver', 'CO', 80201, 98, 5);
   insert into zip_codes values (4, 'Salt Lake City', 'UT', 84101, 98, 4);
   insert into zip_codes values (5, 'Los Angeles', 'CA', 90001, 102, 9);
   insert into zip_codes values (6, 'Houston', 'TX', 77001, 298, 9);
   insert into zip_codes values (7, 'Dallas', 'TX', 75201, 197, 6);
   insert into zip_codes values (8, 'Seattle', 'WA', 98101, 198, 5);
   insert into zip_codes values (9, 'Philadelphia', 'PA', 19019, 187, 8);
   insert into zip_codes values (10, 'Cleveland', 'OH', 44101, 98, 5);
   insert into zip_codes values (11, 'Boston', 'MA', 2101, 127, 5);
   insert into zip_codes values (12, 'Washington', 'DC', 20001, 98, 5);
   insert into zip_codes values (13, 'Miami', 'FL', 33010, 289, 9);
   insert into zip_codes values (14, 'San Francisco', 'CA', 94101, 187, 5);
   insert into zip_codes values (15, 'Atlanta', 'GA', 30301, 98, 5);
   dbms_output.put_line('15 rows added to zip_codes table');
end;
/

create or replace procedure pr_Create_InvFact(
    v_mode            IN  integer,
    v_max_rows        IN  integer,
    v_inv_sparse      IN  number,
    v_so_sparse       IN  number,
    v_po_sparse       IN  number)  
--
--  Jim Maydew  ISMG 6840  6/29/01 - Inventory DW
--  This procedure creates the requested number of fact 
--  table entries based on max rows and sparsity parms
--  
is
    v_i            integer;  -- loop variable
    v_j            integer;  -- loop variable
    v_count        integer;  -- number of fact table rows
    v_nbr_dates    integer;  -- number of dates rows
    v_nbr_custs    integer;  -- number of cust/vendor rows
    v_nbr_branches integer;  -- number of branches
    v_nbr_parts    integer;  -- number of parts
    v_temp         integer;  -- work variable...
    v_temp2        integer;  -- work variable...
    v_est_inv      integer;  -- estimate of inventory rows that will be generated.....
    v_est_so       integer;  -- estimate of sales order rows that will be generated.....
    v_est_po       integer;  -- estimate of purchase order rows that will be generated.....
    v_est_total    integer;  -- combined estimated total of so, po and inventory rows generated
    v_InvKey       inventory_fact.InventoryKey%type;
    v_BPKey        branch_plant_dim.BranchPlantKey%type;
    v_DateKey      date_dim.DateKey%type;
    v_IMKey        item_master_dim.ItemMasterKey%type;
    v_TTKey        trans_type_dim.TransTypeKey%type;
    v_CVKey        cust_vendor_dim.CustVendorKey%type;
    v_ucost        inventory_fact.UnitCost%type;
    v_qty          inventory_fact.Quantity%type;
    v_ecost        inventory_fact.ExtCost%type;
    v_unitCost     inventory_fact.UnitCost%type := 1000;
    v_varUCost     integer := 500;     -- amount that the unit cost can vary either way randomly
    v_quantity     inventory_fact.Quantity%type := 100;
    v_varQty       integer :=50;     -- amount that the quantity can vary either way randomly
begin
--
--  verify mode parms, max rows and sparsity percentages
--
   if (v_mode > 2 or v_mode < 1) then
      raise_application_error(-20071,'Error - Mode must be 1 (estimate) or 2 (generate)');
   end if;
   if (v_inv_sparse <= 0.000) then
      raise_application_error(-20072,'Error - inventory transaction sparsity cannot be zero');
   end if;
   if (v_so_sparse <= 0.000) then
      raise_application_error(-20073,'Error - sales order transaction sparsity cannot be zero');
   end if;
   if (v_po_sparse <= 0.000) then
      raise_application_error(-20074,'Error - purchase order transaction sparsity cannot be zero');
   end if;
--
--  First see if inventory fact table is empty
--
   select count(*) into v_count from inventory_fact;
   if (v_count <> 0) then
      raise_application_error(-20075,'Error - Table (inventory_fact) must be empty - '
       || to_char(v_count) || ' rows exist');
   end if;
--
--  Get count of date rows
--
   select count(*) into v_nbr_dates from date_dim;
   if (v_nbr_dates = 0) then
      raise_application_error(-20076,'Error - Table (date_dim) must NOT be empty - '
       || to_char(v_nbr_dates) || ' rows exist');
   end if;
--
--  Get count of branch plant rows
--
   select count(*) into v_nbr_branches from branch_plant_dim;
   if (v_nbr_branches = 0) then
      raise_application_error(-20077,'Error - Table (branch_plant_dim) must NOT be empty - '
       || to_char(v_nbr_branches) || ' rows exist');
   end if;
--
--  Get count of item master rows
--
   select count(*) into v_nbr_parts from item_master_dim;
   if (v_nbr_parts = 0) then
      raise_application_error(-20078,'Error - Table (item_master_dim) must NOT be empty - '
       || to_char(v_nbr_parts) || ' rows exist');
   end if;
--
--  Get count of customer/vendor rows
--
   select count(*) into v_nbr_custs from cust_vendor_dim;
   if (v_nbr_custs = 0) then
      raise_application_error(-20079,'Error - Table (cust_vendor_dim) must NOT be empty - '
       || to_char(v_nbr_custs) || ' rows exist');
   end if;
--
--  Estimate Rows that will be generated... If estimation mode
--  output details of estimation...
--
   v_est_inv := trunc(v_nbr_dates * v_nbr_branches * v_nbr_parts * 3 * v_inv_sparse);
   v_est_so  := trunc(v_nbr_dates * v_nbr_branches * v_nbr_parts * v_nbr_custs * v_so_sparse);
   v_est_po  := trunc(v_nbr_dates * v_nbr_branches * v_nbr_parts * v_nbr_custs * v_po_sparse);  
   v_est_total := v_est_inv + v_est_so + v_est_po;
--
--  if max rows parameter is off by a factor of 2 or more stop
--
   if (v_est_total > 2*v_max_rows) then
      raise_application_error(-20080,'Max Rows ' || to_char(v_max_rows) ||
          ' is off by a factor > 2 to Estimated Rows ' || to_char(v_est_total));
   end if;
--
--  if mode is estimate then output detailed estimate...
--
   if (v_mode = 1) then
      dbms_output.put_line('**** Estimated Inventory Fact Table Generation **** ');
      dbms_output.put_line('  --------------------------------------------------- ');
      dbms_output.put_line('Table Sizes ');
      dbms_output.put_line('..Rows in Table date_dim        : ' || to_char(v_nbr_dates));
      dbms_output.put_line('..Rows in Table item_master_dim : ' || to_char(v_nbr_parts));
      dbms_output.put_line('..Rows in Table branch_plant_dim: ' || to_char(v_nbr_branches));
      dbms_output.put_line('..Rows in Table cust_vendor_dim : ' || to_char(v_nbr_custs));
      dbms_output.put_line('--------------------------------------------------- ');
      dbms_output.put_line('Supplied Parameters ');
      dbms_output.put_line('..Maximum number of rows to generate: ' || to_char(v_max_rows));
      dbms_output.put_line('..Inventory sparsity percentage     : ' || to_char(v_inv_sparse));
      dbms_output.put_line('..Sales Order sparsity percentage   : ' || to_char(v_so_sparse));
      dbms_output.put_line('..Purchase Order sparsity percentage: ' || to_char(v_po_sparse));
      dbms_output.put_line('--------------------------------------------------- ');
      dbms_output.put_line('Generated Row Estimates '); 
      dbms_output.put_line('..Estimated Number of Inventory Rows      : ' || v_est_inv);
      dbms_output.put_line('..Estimated Number of Sales Order Rows    : ' || v_est_so);
      dbms_output.put_line('..Estimated Number of Purchase Order Rows : ' || v_est_po);
      dbms_output.put_line('..Total Number of Estimated Rows          : ' || v_est_total);
      dbms_output.put_line('    ');
   else
--
--  insert rows into inventory_fact first based on inventory sparsity parm
--
      v_count :=0;
      dbms_random.initialize(1234567);
      for v_i in 1..v_est_inv loop  
         select inventory_seq.nextval into v_InvKey from dual;
         v_BPKey   := mod(abs(dbms_random.random), v_nbr_branches) + 1;
         v_DateKey := mod(abs(dbms_random.random), v_nbr_dates) + 1;
         v_IMKey   := mod(abs(dbms_random.random), v_nbr_parts) + 1;
         v_TTKey   := mod(abs(dbms_random.random), 3) + 1;
--       v_CVKey   := mod(abs(dbms_random.random), v_nbr_custs) + 1;
         v_ucost   := v_unitcost + mod(dbms_random.random, v_varUCost);
         v_qty     := v_quantity + mod(dbms_random.random, v_varQty);
         v_ecost   := round(v_ucost * v_qty, 2);
         insert into inventory_fact values (v_InvKey, v_BPKey, v_DateKey, V_IMKey, v_TTKey, NULL,
            v_ucost, v_qty, v_ecost);
         v_count   := v_count + 1;
--
-- if transaction type is transfer, create related transfer record
--
         if (v_TTkey = 2) then
            select inventory_seq.nextval into v_InvKey from dual;
            v_BPKey :=  mod(abs(dbms_random.random), v_nbr_branches) + 1;
            insert into inventory_fact values (v_InvKey, v_BPKey, v_DateKey, V_IMKey, v_TTKey, NULL,
               v_ucost, -v_qty, -v_ecost);
            v_count := v_count + 1;
         end if;
         if (mod(v_i, trunc(v_est_total/10)) = 0) then
            dbms_output.put_line(' ' || to_char(v_i) || ' inventory facts added to inventory_fact table ');
         end if;
       end loop;  
       dbms_output.put_line(' ' || to_char(v_count) || ' Total inventory facts added to inventory_fact table ');
--
--  insert rows into inventory_fact next based on sales order sparsity parm
--
      for v_i in 1..v_est_so loop  
         select inventory_seq.nextval into v_InvKey from dual;
         v_BPKey   := mod(abs(dbms_random.random), v_nbr_branches) + 1;
         v_DateKey := mod(abs(dbms_random.random), v_nbr_dates) + 1;
         v_IMKey   := mod(abs(dbms_random.random), v_nbr_parts) + 1;
         v_TTKey   := 5;
         v_CVKey   := mod(abs(dbms_random.random), v_nbr_custs) + 1;
         v_ucost   := v_unitcost + mod(dbms_random.random, v_varUCost);
         v_qty     := v_quantity + mod(dbms_random.random, v_varQty);
         v_qty     := v_qty + -1;  -- sales transaction reduce inventory
         v_ecost   := round(v_ucost * v_qty, 2);
         insert into inventory_fact values (v_InvKey, v_BPKey, v_DateKey, V_IMKey, v_TTKey, v_CVKey,
            v_ucost, v_qty, v_ecost);
         v_count := v_count + 1;
         if (mod(v_i, trunc(v_est_total/10)) = 0) then
            dbms_output.put_line(' ' || to_char(v_i) || ' sales order facts added to inventory_fact table ');
         end if;     
       end loop;  
       dbms_output.put_line(' ' || to_char(v_count) || ' Total inventory and sales order facts added to inventory_fact table ');
--
--  insert rows into inventory_fact next based on purchase order sparsity parm
--
      for v_i in 1..v_est_so loop  
         select inventory_seq.nextval into v_InvKey from dual;
         v_BPKey   := mod(abs(dbms_random.random), v_nbr_branches) + 1;
         v_DateKey := mod(abs(dbms_random.random), v_nbr_dates) + 1;
         v_IMKey   := mod(abs(dbms_random.random), v_nbr_parts) + 1;
         v_TTKey   := 4;
         v_CVKey   := mod(abs(dbms_random.random), v_nbr_custs) + 1;
         v_ucost   := v_unitcost + mod(dbms_random.random, v_varUCost);
         v_qty     := v_quantity*100 + mod(dbms_random.random, v_varQty*100);
         v_ecost   := round(v_ucost * v_qty, 2);
         insert into inventory_fact values (v_InvKey, v_BPKey, v_DateKey, V_IMKey, v_TTKey, v_CVKey,
            v_ucost, v_qty, v_ecost);
         v_count := v_count + 1;
         if (mod(v_i, trunc(v_est_total/10)) = 0) then
            dbms_output.put_line(' ' || to_char(v_i) || ' sales order facts added to inventory_fact table ');
         end if;
       end loop;  
   end if;
   dbms_output.put_line(' ' || to_char(v_count) || ' TOTAL inventory, sales and purchasing facts added to inventory_fact table ');
end;
/
